@@ -10,6 +10,7 @@ import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk';
 import Meta from 'gi://Meta';
+import Mtk from 'gi://Mtk';
 import Shell from 'gi://Shell';
 import St from 'gi://St';
 
@@ -17,18 +18,14 @@ import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/ex
 
 import * as DND from 'resource:///org/gnome/shell/ui/dnd.js';
 import * as Main from 'resource:///org/gnome/shell/ui/main.js';
-import * as Overview from 'resource:///org/gnome/shell/ui/overview.js';
 import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
+import {DashItemContainer} from 'resource:///org/gnome/shell/ui/dash.js';
 
-import {WindowPicker, WindowPickerToggle} from './windowPicker.js';
 import {WorkspaceIndicator} from './workspaceIndicator.js';
 
 const ICON_TEXTURE_SIZE = 24;
 const DND_ACTIVATE_TIMEOUT = 500;
 
-const TOOLTIP_OFFSET = 6;
-const TOOLTIP_ANIMATION_TIME = 150;
-
 const GroupingMode = {
     NEVER: 0,
     AUTO: 1,
@@ -123,14 +120,10 @@ class WindowTitle extends St.BoxLayout {
         this._metaWindow = metaWindow;
 
         this._icon = new St.Bin({style_class: 'window-button-icon'});
-        this.add(this._icon);
+        this.add_child(this._icon);
         this.label_actor = new St.Label({y_align: Clutter.ActorAlign.CENTER});
         this.label_actor.clutter_text.single_line_mode = true;
-        this.add(this.label_actor);
-
-        this._textureCache = St.TextureCache.get_default();
-        this._textureCache.connectObject('icon-theme-changed',
-            () => this._updateIcon(), this);
+        this.add_child(this.label_actor);
 
         this._metaWindow.connectObject(
             'notify::wm-class',
@@ -166,20 +159,20 @@ class WindowTitle extends St.BoxLayout {
             this._icon.child = app.create_icon_texture(ICON_TEXTURE_SIZE);
         } else {
             this._icon.child = new St.Icon({
-                icon_name: 'icon-missing',
+                icon_name: 'application-x-executable',
                 icon_size: ICON_TEXTURE_SIZE,
             });
         }
     }
 }
 
-class BaseButton extends St.Button {
+class BaseButton extends DashItemContainer {
     static {
         GObject.registerClass({
             GTypeFlags: GObject.TypeFlags.ABSTRACT,
             Properties: {
                 'ignore-workspace': GObject.ParamSpec.boolean(
-                    'ignore-workspace', 'ignore-workspace', 'ignore-workspace',
+                    'ignore-workspace', null, null,
                     GObject.ParamFlags.READWRITE,
                     false),
             },
@@ -187,12 +180,22 @@ class BaseButton extends St.Button {
     }
 
     constructor(perMonitor, monitorIndex) {
-        super({
+        super();
+
+        this._button = new St.Button({
             style_class: 'window-button',
             can_focus: true,
             x_expand: true,
             button_mask: St.ButtonMask.ONE | St.ButtonMask.THREE,
         });
+        this.setChild(this._button);
+
+        this._button.connect('notify::hover', () => {
+            if (this._button.hover)
+                this.showLabel();
+            else
+                this.hideLabel();
+        });
 
         this._perMonitor = perMonitor;
         this._monitorIndex = monitorIndex;
@@ -200,7 +203,7 @@ class BaseButton extends St.Button {
 
         this.connect('notify::allocation',
             this._updateIconGeometry.bind(this));
-        this.connect('clicked', this._onClicked.bind(this));
+        this._button.connect('clicked', this._onClicked.bind(this));
         this.connect('destroy', this._onDestroy.bind(this));
         this.connect('popup-menu', this._onPopupMenu.bind(this));
 
@@ -217,16 +220,10 @@ class BaseButton extends St.Button {
                 this._windowEnteredOrLeftMonitor.bind(this),
                 this);
         }
-
-        this._tooltip = new Tooltip(this, {
-            style_class: 'dash-label',
-            visible: false,
-        });
-        Main.uiGroup.add_child(this._tooltip);
     }
 
     get active() {
-        return this.has_style_class_name('focused');
+        return this._button.has_style_class_name('focused');
     }
 
     // eslint-disable-next-line camelcase
@@ -245,6 +242,18 @@ class BaseButton extends St.Button {
         this._updateVisibility();
     }
 
+    showLabel() {
+        const [, , preferredTitleWidth] = this.label_actor.get_preferred_size();
+        const maxTitleWidth = this.label_actor.allocation.get_width();
+        const isTitleFullyShown = preferredTitleWidth <= maxTitleWidth;
+
+        const labelText = isTitleFullyShown
+            ? '' : this.label_actor.text;
+
+        this.setLabelText(labelText);
+        super.showLabel();
+    }
+
     _setLongPressTimeout() {
         if (this._longPressTimeoutId)
             return;
@@ -267,24 +276,24 @@ class BaseButton extends St.Button {
         delete this._longPressTimeoutId;
     }
 
-    vfunc_button_press_event(buttonEvent) {
-        if (buttonEvent.button === 1)
+    vfunc_button_press_event(event) {
+        if (event.get_button() === 1)
             this._setLongPressTimeout();
-        return super.vfunc_button_press_event(buttonEvent);
+        return super.vfunc_button_press_event(event);
     }
 
-    vfunc_button_release_event(buttonEvent) {
+    vfunc_button_release_event(event) {
         this._removeLongPressTimeout();
 
-        return super.vfunc_button_release_event(buttonEvent);
+        return super.vfunc_button_release_event(event);
     }
 
-    vfunc_touch_event(touchEvent) {
-        if (touchEvent.type === Clutter.EventType.TOUCH_BEGIN)
+    vfunc_touch_event(event) {
+        if (event.type() === Clutter.EventType.TOUCH_BEGIN)
             this._setLongPressTimeout();
-        else if (touchEvent.type === Clutter.EventType.TOUCH_END)
+        else if (event.type() === Clutter.EventType.TOUCH_END)
             this._removeLongPressTimeout();
-        return super.vfunc_touch_event(touchEvent);
+        return super.vfunc_touch_event(event);
     }
 
     activate() {
@@ -345,9 +354,9 @@ class BaseButton extends St.Button {
 
     _updateStyle() {
         if (this._isFocused())
-            this.add_style_class_name('focused');
+            this._button.add_style_class_name('focused');
         else
-            this.remove_style_class_name('focused');
+            this._button.remove_style_class_name('focused');
     }
 
     _windowEnteredOrLeftMonitor(_metaDisplay, _monitorIndex, _metaWindow) {
@@ -369,7 +378,7 @@ class BaseButton extends St.Button {
     }
 
     _getIconGeometry() {
-        let rect = new Meta.Rectangle();
+        const rect = new Mtk.Rectangle();
 
         [rect.x, rect.y] = this.get_transformed_position();
         [rect.width, rect.height] = this.get_transformed_size();
@@ -383,7 +392,6 @@ class BaseButton extends St.Button {
     }
 
     _onDestroy() {
-        this._tooltip.destroy();
     }
 }
 
@@ -396,15 +404,17 @@ class WindowButton extends BaseButton {
         super(perMonitor, monitorIndex);
 
         this.metaWindow = metaWindow;
+        this._unmanaging = false;
         metaWindow.connectObject(
             'notify::skip-taskbar', () => this._updateVisibility(),
             'workspace-changed', () => this._updateVisibility(),
+            'unmanaging', () => (this._unmanaging = true),
             this);
 
         this._updateVisibility();
 
         this._windowTitle = new WindowTitle(this.metaWindow);
-        this.set_child(this._windowTitle);
+        this._button.set_child(this._windowTitle);
         this.label_actor = this._windowTitle.label_actor;
 
         this._contextMenu = new WindowContextMenu(this, this.metaWindow);
@@ -412,7 +422,7 @@ class WindowButton extends BaseButton {
             this._onMenuStateChanged.bind(this));
         this._contextMenu.actor.hide();
         this._contextMenuManager.addMenu(this._contextMenu);
-        Main.uiGroup.add_actor(this._contextMenu.actor);
+        Main.uiGroup.add_child(this._contextMenu.actor);
 
         global.display.connectObject('notify::focus-window',
             () => this._updateStyle(), this);
@@ -439,9 +449,9 @@ class WindowButton extends BaseButton {
         super._updateStyle();
 
         if (this.metaWindow.minimized)
-            this.add_style_class_name('minimized');
+            this._button.add_style_class_name('minimized');
         else
-            this.remove_style_class_name('minimized');
+            this._button.remove_style_class_name('minimized');
     }
 
     _windowEnteredOrLeftMonitor(metaDisplay, monitorIndex, metaWindow) {
@@ -450,6 +460,9 @@ class WindowButton extends BaseButton {
     }
 
     _updateVisibility() {
+        if (this._unmanaging)
+            return;
+
         this.visible = this._isWindowVisible(this.metaWindow);
     }
 
@@ -533,30 +546,30 @@ class AppButton extends BaseButton {
         this._updateVisibility();
 
         let stack = new St.Widget({layout_manager: new Clutter.BinLayout()});
-        this.set_child(stack);
+        this._button.set_child(stack);
 
         this._singleWindowTitle = new St.Bin({
             x_expand: true,
         });
-        stack.add_actor(this._singleWindowTitle);
+        stack.add_child(this._singleWindowTitle);
 
         this._multiWindowTitle = new St.BoxLayout({
             style_class: 'window-button-box',
             x_expand: true,
         });
-        stack.add_actor(this._multiWindowTitle);
+        stack.add_child(this._multiWindowTitle);
 
         this._icon = new St.Bin({
             style_class: 'window-button-icon',
             child: app.create_icon_texture(ICON_TEXTURE_SIZE),
         });
-        this._multiWindowTitle.add(this._icon);
+        this._multiWindowTitle.add_child(this._icon);
 
         let label = new St.Label({
             text: app.get_name(),
             y_align: Clutter.ActorAlign.CENTER,
         });
-        this._multiWindowTitle.add(label);
+        this._multiWindowTitle.add_child(label);
         this._multiWindowTitle.label_actor = label;
 
         this._menuManager = new PopupMenu.PopupMenuManager(this);
@@ -566,18 +579,13 @@ class AppButton extends BaseButton {
         this._menu.actor.hide();
         this._menu.connect('activate', this._onMenuActivate.bind(this));
         this._menuManager.addMenu(this._menu);
-        Main.uiGroup.add_actor(this._menu.actor);
+        Main.uiGroup.add_child(this._menu.actor);
 
         this._appContextMenu = new AppContextMenu(this);
         this._appContextMenu.connect('open-state-changed',
             this._onMenuStateChanged.bind(this));
         this._appContextMenu.actor.hide();
-        Main.uiGroup.add_actor(this._appContextMenu.actor);
-
-        this._textureCache = St.TextureCache.get_default();
-        this._textureCache.connectObject('icon-theme-changed', () => {
-            this._icon.child = app.create_icon_texture(ICON_TEXTURE_SIZE);
-        }, this);
+        Main.uiGroup.add_child(this._appContextMenu.actor);
 
         this.app.connectObject('windows-changed',
             () => this._windowsChanged(), this);
@@ -637,7 +645,7 @@ class AppButton extends BaseButton {
                 this._windowContextMenu = new WindowContextMenu(this, this.metaWindow);
                 this._windowContextMenu.connect(
                     'open-state-changed', this._onMenuStateChanged.bind(this));
-                Main.uiGroup.add_actor(this._windowContextMenu.actor);
+                Main.uiGroup.add_child(this._windowContextMenu.actor);
                 this._windowContextMenu.actor.hide();
                 this._contextMenuManager.addMenu(this._windowContextMenu);
             }
@@ -682,7 +690,7 @@ class AppButton extends BaseButton {
                 for (let i = 0; i < windows.length; i++) {
                     let windowTitle = new WindowTitle(windows[i]);
                     let item = new PopupMenu.PopupBaseMenuItem();
-                    item.add_actor(windowTitle);
+                    item.add_child(windowTitle);
                     item._window = windows[i];
                     this._menu.addMenuItem(item);
                 }
@@ -728,36 +736,26 @@ class WindowList extends St.Widget {
         this._monitor = monitor;
 
         let box = new St.BoxLayout({x_expand: true, y_expand: true});
-        this.add_actor(box);
+        this.add_child(box);
 
-        let toggle = new WindowPickerToggle();
-        box.add_actor(toggle);
-
-        toggle.connect('notify::checked',
-            this._updateWindowListVisibility.bind(this));
-
-        let layout = new Clutter.BoxLayout({homogeneous: true});
-        this._windowList = new St.Widget({
+        this._windowList = new St.BoxLayout({
             style_class: 'window-list',
             reactive: true,
-            layout_manager: layout,
             x_align: Clutter.ActorAlign.START,
             x_expand: true,
             y_expand: true,
         });
         box.add_child(this._windowList);
 
-        this._windowList.connect('style-changed', () => {
-            let node = this._windowList.get_theme_node();
-            let spacing = node.get_length('spacing');
-            this._windowList.layout_manager.spacing = spacing;
-        });
         this._windowList.connect('scroll-event', this._onScrollEvent.bind(this));
 
         let indicatorsBox = new St.BoxLayout({x_align: Clutter.ActorAlign.END});
-        box.add(indicatorsBox);
+        box.add_child(indicatorsBox);
 
-        this._workspaceIndicator = new WorkspaceIndicator();
+        this._workspaceIndicator = new BottomWorkspaceIndicator({
+            baseStyleClass: 'window-list-workspace-indicator',
+            settings,
+        });
         indicatorsBox.add_child(this._workspaceIndicator.container);
 
         this._mutterSettings = new Gio.Settings({schema_id: 'org.gnome.mutter'});
@@ -770,7 +768,9 @@ class WindowList extends St.Widget {
         this._updateWorkspaceIndicatorVisibility();
 
         this._menuManager = new PopupMenu.PopupMenuManager(this);
-        this._menuManager.addMenu(this._workspaceIndicator.menu);
+        this._workspaceIndicator.connectObject('menu-set',
+            () => this._onWorkspaceMenuSet(), this);
+        this._onWorkspaceMenuSet();
 
         Main.layoutManager.addChrome(this, {
             affectsStruts: true,
@@ -829,7 +829,7 @@ class WindowList extends St.Widget {
 
         this._windowSignals = new Map();
         global.display.connectObject(
-            'window-created', (dsp, win) => this._addWindow(win), this);
+            'window-created', (dsp, win) => this._addWindow(win, true), this);
 
         Main.xdndHandler.connectObject(
             'drag-begin', () => this._monitorDrag(),
@@ -867,6 +867,11 @@ class WindowList extends St.Widget {
         children[newActive].activate();
     }
 
+    _onWorkspaceMenuSet() {
+        if (this._workspaceIndicator.menu)
+            this._menuManager.addMenu(this._workspaceIndicator.menu);
+    }
+
     _updatePosition() {
         this.set_position(
             this._monitor.x,
@@ -883,20 +888,6 @@ class WindowList extends St.Widget {
         this._workspaceIndicator.visible = hasWorkspaces && workspacesOnMonitor;
     }
 
-    _updateWindowListVisibility() {
-        const {windowPicker} = Extension.lookupByURL(import.meta.url);
-        const visible = !windowPicker.visible;
-
-        this._windowList.ease({
-            opacity: visible ? 255 : 0,
-            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-            duration: Overview.ANIMATION_TIME,
-        });
-
-        this._windowList.reactive = visible;
-        this._windowList.get_children().forEach(c => (c.reactive = visible));
-    }
-
     _getPreferredUngroupedWindowListWidth() {
         if (this._windowList.get_n_children() === 0)
             return this._windowList.get_preferred_width(-1)[1];
@@ -955,14 +946,14 @@ class WindowList extends St.Widget {
                        w2.metaWindow.get_stable_sequence();
             });
             for (let i = 0; i < windows.length; i++)
-                this._addWindow(windows[i].metaWindow);
+                this._addWindow(windows[i].metaWindow, false);
         } else {
             let apps = this._appSystem.get_running().sort((a1, a2) => {
                 return _getAppStableSequence(a1) -
                        _getAppStableSequence(a2);
             });
             for (let i = 0; i < apps.length; i++)
-                this._addApp(apps[i]);
+                this._addApp(apps[i], false);
         }
     }
 
@@ -976,26 +967,26 @@ class WindowList extends St.Widget {
             return;
 
         if (app.state === Shell.AppState.RUNNING)
-            this._addApp(app);
+            this._addApp(app, true);
         else if (app.state === Shell.AppState.STOPPED)
             this._removeApp(app);
     }
 
-    _addApp(app) {
+    _addApp(app, animate) {
         let button = new AppButton(app, this._perMonitor, this._monitor.index);
         this._settings.bind('display-all-workspaces',
             button, 'ignore-workspace', Gio.SettingsBindFlags.GET);
         this._windowList.add_child(button);
+        button.show(animate);
     }
 
     _removeApp(app) {
         let children = this._windowList.get_children();
         let child = children.find(c => c.app === app);
-        if (child)
-            child.destroy();
+        child?.animateOutAndDestroy();
     }
 
-    _addWindow(win) {
+    _addWindow(win, animate) {
         if (!this._grouped)
             this._checkGrouping();
 
@@ -1013,6 +1004,7 @@ class WindowList extends St.Widget {
         this._settings.bind('display-all-workspaces',
             button, 'ignore-workspace', Gio.SettingsBindFlags.GET);
         this._windowList.add_child(button);
+        button.show(animate);
     }
 
     _removeWindow(win) {
@@ -1029,8 +1021,7 @@ class WindowList extends St.Widget {
 
         let children = this._windowList.get_children();
         let child = children.find(c => c.metaWindow === win);
-        if (child)
-            child.destroy();
+        child?.animateOutAndDestroy();
     }
 
     _monitorDrag() {
@@ -1101,12 +1092,33 @@ class WindowList extends St.Widget {
     }
 }
 
+class BottomWorkspaceIndicator extends WorkspaceIndicator {
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(params) {
+        super(params);
+
+        this.remove_style_class_name('panel-button');
+    }
+
+    setMenu(menu) {
+        super.setMenu(menu);
+
+        if (!menu)
+            return;
+
+        this.menu.actor.updateArrowSide(St.Side.BOTTOM);
+        this.menu.actor.remove_style_class_name('panel-menu');
+    }
+}
+
 export default class WindowListExtension extends Extension {
     constructor(metadata) {
         super(metadata);
 
         this._windowLists = null;
-        this._hideOverviewOrig = Main.overview.hide;
     }
 
     enable() {
@@ -1119,13 +1131,6 @@ export default class WindowListExtension extends Extension {
         Main.layoutManager.connectObject('monitors-changed',
             () => this._buildWindowLists(), this);
 
-        this.windowPicker = new WindowPicker();
-
-        Main.overview.hide = () => {
-            this.windowPicker.close();
-            this._hideOverviewOrig.call(Main.overview);
-        };
-
         this._buildWindowLists();
     }
 
@@ -1154,78 +1159,9 @@ export default class WindowListExtension extends Extension {
             windowList.destroy();
         });
         this._windowLists = null;
-
-        this.windowPicker.destroy();
-        delete this.windowPicker;
-
-        Main.overview.hide = this._hideOverviewOrig;
     }
 
     someWindowListContains(actor) {
         return this._windowLists.some(list => list.contains(actor));
     }
 }
-
-class Tooltip extends St.Label {
-    static {
-        GObject.registerClass(this);
-    }
-
-    constructor(widget, params) {
-        super(params);
-
-        this._widget = widget;
-
-        this._widget.connect('notify::hover', () => {
-            if (this._widget.hover)
-                this.open();
-            else
-                this.close();
-        });
-    }
-
-    open() {
-        const buttonTitleWidget = this._widget.label_actor;
-        const [, , preferredTitleWidth] = buttonTitleWidget.get_preferred_size();
-        const maxTitleWidth = buttonTitleWidget.allocation.get_width();
-        const isTitleFullyShown = preferredTitleWidth <= maxTitleWidth;
-
-        if (isTitleFullyShown)
-            return;
-
-        this.set({
-            text: this._widget.label_actor.get_text(),
-            visible: true,
-            opacity: 0,
-        });
-
-        const [stageX, stageY] = this._widget.get_transformed_position();
-        const thumbWidth = this._widget.allocation.get_width();
-        const tipWidth = this.width;
-        const tipHeight = this.height;
-        const xOffset = Math.floor((thumbWidth - tipWidth) / 2);
-        const monitor = Main.layoutManager.findMonitorForActor(this);
-        const x = Math.clamp(
-            stageX + xOffset,
-            monitor.x,
-            monitor.x + monitor.width - tipWidth);
-        const y = stageY - tipHeight - TOOLTIP_OFFSET;
-        this.set_position(x, y);
-
-        this.ease({
-            opacity: 255,
-            duration: TOOLTIP_ANIMATION_TIME,
-            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-            onComplete: () => (this.visible = this._widget.hover),
-        });
-    }
-
-    close() {
-        this.ease({
-            opacity: 0,
-            duration: TOOLTIP_ANIMATION_TIME,
-            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
-            onComplete: () => (this.visible = this._widget.hover),
-        });
-    }
-}
